<?php
/**
 *
 * @since             0.0.1
 * @package           Omni
 *
 * @wordpress-plugin
 * Plugin Name:       Omni
 * Version:           0.19.2
 * Author:            Proxima
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       omni
 */

// If this file is called directly, abort.
if (!defined("WPINC")) {
    die();
}

// Generate and save API key when plugin is activated
function omni_reorder_activate()
{
    define("GET_TABLES", check_and_get_tables());

    // Ensure that the WordPress function get_file_data is available
    if (!function_exists("get_file_data")) {
        require_once ABSPATH . "wp-admin/includes/file.php";
    }

    // Get plugin data
    $plugin_data = get_file_data(__FILE__, ["Version" => "Version"]);
    $version = $plugin_data["Version"];

    // Define the version
    define("OMNI_VERSION", $version);

    if (!get_option("omni_api_key")) {
        $api_key = wp_generate_password(20, false);
        update_option("omni_api_key", $api_key);
    }
    if (
        !get_option("omni_table_file_paths") ||
        !is_array(get_option("omni_table_file_paths"))
    ) {
        $table = [];
        update_option("omni_table_file_paths", $table);
    }
    if (get_option("omni_backend_url") === false) {
        update_option(
            "omni_backend_url",
            "https://click-aggregator-391011.ew.r.appspot.com/"
        );
    } elseif (
        get_option("omni_backend_url") ===
        "https://omni-backend.yaronlev.repl.co/"
    ) {
        update_option(
            "omni_backend_url",
            "https://click-aggregator-391011.ew.r.appspot.com/"
        );
    }

    update_html_files_with_new_script();
}

add_action("init", "omni_reorder_activate");

add_action("admin_menu", "omni_reorder_add_menu");

function omni_reorder_add_menu()
{
    // Add the top-level menu item for 'Omni'
    add_menu_page(
        "Omni", // Page title
        "Omni", // Menu title (displayed in the sidebar)
        "manage_options", // Capability
        "omni-connect-pages", // Menu slug
        "omni_reorder_settings_page", // Function to display the page
        "dashicons-list-view", // Icon
        50 // Position
    );

    // Add 'Connect pages to Omni' submenu under 'Omni'
    add_submenu_page(
        "omni-connect-pages", // Parent slug (matches top-level menu slug)
        "Connect pages to Omni", // Page title
        "Connect pages to Omni", // Menu title
        "manage_options", // Capability
        "omni-connect-pages", // Menu slug
        "omni_reorder_settings_page" // Function to display the page
    );

    // Add 'Settings' submenu under 'Omni'
    add_submenu_page(
        "omni-connect-pages", // Parent slug (matches top-level menu slug)
        "Settings", // Page title
        "Settings", // Menu title
        "manage_options", // Capability
        "omni-settings", // Menu slug
        "omni_wp_settings_page" // Function to display the page
    );

    // Add 'Ordered Pages' submenu under 'Omni'
    add_submenu_page(
        "omni-reorder.php", // Parent slug
        "Ordered Pages", // Page title
        "Ordered Pages", // Menu title
        "manage_options", // Capability
        "ordered-pages", // Menu slug
        "omni_display_ordered_pages" // Function to display the page
    );
}

function omni_display_ordered_pages()
{
    // Enqueue jQuery UI Sortable
    wp_enqueue_script("jquery-ui-sortable");

    // Check if reorder-page.js exists and then enqueue
    $reorder_page_script_path =
        plugin_dir_path(__FILE__) . "scripts/reorder-page.js";
    if (file_exists($reorder_page_script_path)) {
        wp_enqueue_script(
            "reorder-page",
            plugins_url("scripts/reorder-page.js", __FILE__),
            ["jquery", "jquery-ui-sortable"],
            "1.0",
            true
        );
    }

    // Pass API URL and API Key to JavaScript
    $api_url = rest_url("reorder/reorder");
    $api_key = get_option("omni_api_key");
    wp_localize_script("reorder-page", "omniSettings", [
        "apiKey" => $api_key,
        "apiUrl" => $api_url,
    ]);

    // Enqueue Font Awesome
    wp_enqueue_style(
        "font-awesome",
        "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.3/css/all.min.css"
    );

    // Check if a specific page ID is set in the URL parameters
    if (isset($_GET["page_id"])) {
        $page_id = $_GET["page_id"];
        $body = make_api_request("get-order/" . $page_id);
        $page = json_decode($body, true);

        // Check if the page ID exists in the page
        if (isset($page["id"])) {
            // Start output buffering
            ob_start();
            // Start of HTML
            ?>
            <div style="width: 60%; margin: 0 auto; font-size: 16px; margin-top: 8rem">
                <h2 style="text-align: center; color: #333; margin-bottom: 2rem">Ordered Pages for Page ID: <?php echo $page_id; ?></h2>
                <form method="post" action="">
          <input class="pageId" type="hidden" name="pageId[]" value="<?php echo $page_id; ?>" />
                    <ul id="sortableTable" style="list-style: none; padding: 0;">
                        <?php foreach (
                            $page["order"]
                            as $order_number => $item
                        ) { ?>
                            <li class="sortableRow" style="margin-bottom: 10px; background: #f9f9f9; padding: 10px; border-radius: 5px; display: flex; align-items: center; justify-content: space-between;">
                                <div>
									<input type="checkbox" class="itemVisibilityCheckbox" name="item_visibility[<?php echo $item[
             "id"
         ]; ?>]" <?php echo !isset($item["hidden"]) || !$item["hidden"]
    ? "checked"
    : ""; ?>>
                                    <strong class="title"><?php echo $item[
                                        "title"
                                    ]; ?></strong>
                  <?php if (isset($item["id"])) { ?>
                   (ID: <span class="id"><?php echo $item["id"]; ?></span>)
                                    <input class="order" type="hidden" name="order[]" value="<?php echo $order_number; ?>" />
          <input class="id" type="hidden" name="id[]" value="<?php echo $item[
              "id"
          ]; ?>" />				
                  <?php } ?>

                                    <input class="title" type="hidden" name="title[]" value="<?php echo $item[
                                        "title"
                                    ]; ?>" />
                                </div>
                                <div>
                                    <span class="orderIndex"><?php echo $order_number; ?></span>
                  <i class="fas fa-sort"></i>
                                </div>
                            </li>
                        <?php } ?>
                    </ul>
                    <input type="submit" value="Save Order" style="cursor: pointer; background: #28a745; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; transition: opacity 0.3s ease-in-out;" onmouseover="this.style.opacity=0.75;" onmouseout="this.style.opacity=1;">
                    <input type="button" id="resetOrder" value="Reset changes" style="cursor: pointer; float: right; background: #dc3545; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; transition: opacity 0.3s ease-in-out;" onmouseover="this.style.opacity=0.75;" onmouseout="this.style.opacity=1;">
                </form>
            </div>
            <?php
            // End of HTML

            $output = ob_get_clean(); // Get the output and clean the buffer
            echo $output; // Echo the output

        }
    }
}

function omni_reorder_settings_page()
{
    $current_paths = GET_TABLES;
    // Retrieve the current API key
    $api_key = get_option("omni_api_key");

    $alert_message = "";

    // Handle form submission for page settings
    if ($_SERVER["REQUEST_METHOD"] === "POST") {
        // Handle WordPress page selection
        if (isset($_POST["omni_page_select"])) {
            $selected_page_id = $_POST["omni_page_select"];
            $selected_page_title = get_the_title($selected_page_id);
            $selected_page_url = get_permalink($selected_page_id);

            $selected_page_url = preg_replace(
                "#^https?://#",
                "",
                $selected_page_url
            );

            $page = [
                "page" => $selected_page_title,
                "id" => $selected_page_id,
                "type" => "wp",
                "url" => $selected_page_url,
            ];

            if (!empty($current_paths)) {
                $current_paths[$selected_page_id] = $page;
            } else {
                $current_paths = [];
                $current_paths[$selected_page_id] = $page;
            }

            update_option("omni_table_file_paths", $current_paths, false);
        }
        // Handle HTML file path addition
        elseif (isset($_POST["html_file_path"])) {
            $html_file_url = sanitize_text_field($_POST["html_file_path"]);

            if (empty($html_file_url)) {
                $alert_message = "Please enter a valid HTML file path.";
            } else {
                // Convert URL to server path
                $site_url = get_site_url() . "/"; // e.g., https://mediabiro.com
                $base_path = ABSPATH; // e.g., /home/mediabir/public_html/
                $html_file_path = str_replace(
                    $site_url,
                    $base_path,
                    $html_file_url
                );

                if (!file_exists($html_file_path)) {
                    $alert_message =
                        "The file at the specified path does not exist.";
                } elseif (!isHtmlFile($html_file_path)) {
                    $alert_message = "The specified file is not an HTML file.";
                } else {
                    // Extract file information
                    $path_parts = pathinfo($html_file_path);

                    // Define the backups folder path
                    $backups_folder_path = $path_parts["dirname"] . "/backups";

                    // Create the 'backups' folder if it doesn't exist
                    if (
                        !file_exists($backups_folder_path) &&
                        !mkdir($backups_folder_path, 0777, true)
                    ) {
                        error_log(
                            "Failed to create backups folder: $backups_folder_path"
                        );
                    }

                    // Generate a timestamped backup file name
                    $timestamp = date("Y-m-d_H-i-s");
                    $backup_file_name =
                        $path_parts["filename"] .
                        "-" .
                        $timestamp .
                        "." .
                        $path_parts["extension"];
                    $backup_file_path =
                        $backups_folder_path . "/" . $backup_file_name;

                    // Copy the original file to the new backup file
                    if (!copy($html_file_path, $backup_file_path)) {
                        error_log(
                            "Failed to copy $html_file_path to $backup_file_path"
                        );
                    }

                    // Read the contents of the original HTML file
                    $html_content = file_get_contents($html_file_path);

                    // Check if jQuery script already exists
                    if (
                        strpos(
                            $html_content,
                            '<script src="https://code.jquery.com/jquery'
                        ) === false
                    ) {
                        // Insert the latest jQuery script tag before the closing </head> tag
                        $jqueryScript =
                            '<script src="https://code.jquery.com/jquery-latest.min.js" crossorigin="anonymous"></script>';
                        $html_content = str_replace(
                            "</head>",
                            $jqueryScript . "\n</head>",
                            $html_content
                        );
                    }

                    // Set a global JavaScript variable for use in html-file-script.js
                    $globalVarScript =
                        '<script>window.homeUrl = "' .
                        home_url() .
                        '";</script>';

                    // Check if the custom script already exists
                    $pluginUrl = plugins_url(
                        "scripts/html-file-script.js",
                        __FILE__
                    );
                    $customScriptTag =
                        '<script src="' . $pluginUrl . '" async></script>';
                    if (strpos($html_content, $customScriptTag) === false) {
                        // Insert the custom script tag before the closing </body> tag
                        $html_content = str_replace(
                            "</body>",
                            $globalVarScript .
                                "\n" .
                                $customScriptTag .
                                "\n</body>",
                            $html_content
                        );
                    }

                    // Save the modified HTML back to the original file
                    file_put_contents($html_file_path, $html_content);

                    $html_file_id = "html_" . md5($html_file_path); // Creating a unique ID for the HTML file

                    // Determine if the input is a URL or a file path
                    if (!filter_var($html_file_url, FILTER_VALIDATE_URL)) {
                        // It's a file path
                        $html_file_url =
                            get_site_url() .
                            "/" .
                            str_replace($base_path, "", $html_file_url); // Convert file path to URL
                    }

                    $html_file_url = preg_replace(
                        "#^https?://#",
                        "",
                        $html_file_url
                    );

                    $current_paths[$html_file_id] = [
                        "page" => $html_file_path, // Save server path
                        "id" => $html_file_id,
                        "type" => "html",
                        "url" => $html_file_url, // Save the URL
                    ];

                    update_option(
                        "omni_table_file_paths",
                        $current_paths,
                        false
                    );
                }
            }
        }
        // Handle path removal
        elseif (isset($_POST["remove_path"])) {
            $remove_page_id = sanitize_text_field($_POST["remove_path"]);

            if (isset($current_paths[$remove_page_id])) {
                $file_path = $current_paths[$remove_page_id]["page"];

                // Ensure the file is an HTML file
                if (pathinfo($file_path, PATHINFO_EXTENSION) === "html") {
                    // Read the contents of the HTML file
                    $html_content = file_get_contents($file_path);

                    // Remove the jQuery script if it exists
                    $jqueryScript =
                        '<script src="https://code.jquery.com/jquery-latest.min.js" crossorigin="anonymous"></script>';
                    $html_content = str_replace(
                        $jqueryScript . "\n",
                        "",
                        $html_content
                    );

                    // Remove the global variable script if it exists
                    $globalVarScript =
                        '<script>window.homeUrl = "' .
                        home_url() .
                        '";</script>';
                    $html_content = str_replace(
                        $globalVarScript . "\n",
                        "",
                        $html_content
                    );

                    // Remove the custom script tag if it exists
                    $pluginUrl = plugins_url(
                        "scripts/html-file-script.js",
                        __FILE__
                    );
                    $customScriptTag =
                        '<script src="' . $pluginUrl . '" async></script>';
                    $html_content = str_replace(
                        $customScriptTag . "\n",
                        "",
                        $html_content
                    );

                    // Save the modified HTML back to the original file
                    file_put_contents($file_path, $html_content);
                }

                // Remove the path from the array and update the option
                unset($current_paths[$remove_page_id]);
                update_option("omni_table_file_paths", $current_paths, false);
            }
        }
    }

    ob_start();
    ?>
     <!-- Start of HTML Output -->
    <div style="width: 80%; margin: 0 auto; font-size: 16px; margin-top: 8rem">

    <?php if (!empty($current_paths)): ?>
    <h2 style="text-align: center; color: #333; margin-bottom: 2rem">Connected Pages and HTML Files</h2>
        <ul style="list-style: none; padding: 0; margin-bottom: 4rem">
            <?php foreach ($current_paths as $id => $path): ?>
                <li style="margin-bottom: 10px; background: #f9f9f9; padding: 10px; border-radius: 5px; display: flex; align-items: center; justify-content: space-between;">
                    <div>
                        <?php if ($path["type"] === "wp"): ?>
                            <strong><?= esc_html(
                                $path["page"]
                            ) ?></strong> (ID: <?= esc_html($path["id"]) ?>)
                        <?php else: ?>
                            HTML File: <strong><?= esc_html(
                                $path["page"]
                            ) ?></strong>
                        <?php endif; ?>
                    </div>
                    <div>
             <a href="<?= esc_url($path["url"]) ?>" title="<?= esc_url(
    $path["url"]
) ?>" target="_blank" style="display: inline-block; cursor: pointer; background: #28a745; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; text-decoration: none; margin-right: 10px; font-size: 14px;">Open Page</a>
                        <form method="post" style="display:inline;">
                            <input type="hidden" name="remove_path" value="<?= esc_attr(
                                $id
                            ) ?>">
                            <input type="submit" value="Remove" style="cursor: pointer; background: #dc3545; color: #fff; border: none; padding: 5px 10px; border-radius: 5px;">
                        </form>
                        <form method="post" action="<?php echo admin_url(
                            "admin.php?page=ordered-pages&page_id=" .
                                $path["id"]
                        ); ?>" style="display:inline;">
                            <input type="hidden" name="edit_path" value="<?= esc_attr(
                                $id
                            ) ?>">
                            <input type="submit" value="Edit" style="cursor: pointer; background: #007bff; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; margin-left: 10px;">
                        </form>
                    </div>
                </li>
            <?php endforeach; ?>
        </ul>
    <?php endif; ?>

    <!-- HTML for adding a WordPress page -->
    <h2 style="text-align: center; color: #333; margin-top: 20px; margin-bottom: 2rem">Add Page</h2>
     <div style="display: flex; justify-content: center; align-items: center; width: 100%;">
    <form method="post" style="width: 60%; display: flex; justify-content: center; align-items: center; gap: 10px; margin-bottom: 4rem;">
      <select name="omni_page_select" style="padding: 5px; border-radius: 5px; flex-grow: 1;">
        <option value="" disabled selected hidden>Select a page</option>
        <?php
        $pages = get_pages();
        foreach ($pages as $page) {
            $is_selected = array_key_exists($page->ID, $current_paths);
            if (!$is_selected) {
                echo '<option value="' .
                    esc_attr($page->ID) .
                    '">' .
                    esc_html($page->post_title) .
                    "</option>";
            }
        }
        ?>
      </select>
      <input type="submit" value="Add Page" style="cursor: pointer; background: #28a745; color: #fff; border: none; padding: 5px 10px; border-radius: 5px;">
    </form>
</div>


    <!-- HTML for adding an HTML file path -->
    <h2 style="text-align: center; color: #333; margin-top: 20px; margin-bottom: 2rem">Add HTML File Path</h2>
    <div style="display: flex; justify-content: center; align-items: center; width: 100%;">
    <form method="post" style="width: 60%; display: flex; justify-content: center; align-items: center; gap: 10px;">
      <input type="text" name="html_file_path" style="padding: 5px; border-radius: 5px; width: 100%;" placeholder="Enter HTML file path or site URL">
      <input type="submit" value="Add HTML File" style="cursor: pointer; background: #28a745; color: #fff; border: none; padding: 5px 10px; border-radius: 5px;">
    </form>
    </div >
    <p style="text-align: center; display: flex; justify-content: center; align-items: center; width: 100%;  margin-top: 2rem">
       Please specify the exact path or URL to the html file on your server, including the file name. Example: 
      <br>
      <br>
      <?php echo ABSPATH . "path/to/the/page.html"; ?>
      <br>
      or 
      <br>
      <?php echo get_site_url() . "/path/to/the/page.html"; ?>
    </p>

    </div>
  <?php if (!empty($alert_message)): ?>
    <script>
      alert('<?= addslashes($alert_message) ?>');
    </script>
  <?php endif; ?>
    <!-- End of HTML Output -->
    <?php
    $output = ob_get_clean();
    echo $output;
}

function isHtmlFile($filePath)
{
    $fileExtension = strtolower(pathinfo($filePath, PATHINFO_EXTENSION));
    return $fileExtension === "html" || $fileExtension === "htm";
}

add_action("rest_api_init", function () {
    register_rest_route("reorder", "/reorder", [
        "methods" => "POST",
        "callback" => "reorder_table",
        "permission_callback" => function ($request) {
            return omni_verify_api_key($request);
        },
    ]);

    register_rest_route("reorder", "/remove-page", [
        "methods" => "POST",
        "callback" => "remove_page",
        "permission_callback" => function ($request) {
            return omni_verify_api_key($request);
        },
    ]);

    register_rest_route("reorder", "/validate-api", [
        "methods" => "POST",
        "callback" => "validate_api_key",
        "permission_callback" => "__return_true", // You can add authentication if needed
    ]);

    register_rest_route("reorder", "get-tables", [
        "methods" => WP_REST_Server::READABLE,
        "callback" => "get_tables",
        "permission_callback" => "__return_true", // Remove this line to require authentication.
    ]);

    register_rest_route("reorder", "get-order", [
        "methods" => WP_REST_Server::READABLE,
        "callback" => "get_order",
        "permission_callback" => "__return_true", // Remove this line to require authentication.
    ]);

    register_rest_route("reorder", "/get-order/(?P<id>[a-zA-Z0-9_]+)", [
        "methods" => WP_REST_Server::READABLE,
        "callback" => "get_order_by_id",
        "args" => [
            "id" => [
                "validate_callback" => function ($param, $request, $key) {
                    return preg_match('/^[a-zA-Z0-9_]+$/', $param); // Validate the id as alphanumeric with underscores
                },
            ],
        ],
        "permission_callback" => "__return_true", // Remove this line to require authentication.
    ]);

    register_rest_route("reorder", "test-master", [
        "methods" => WP_REST_Server::READABLE,
        "callback" => "test_master",
        "permission_callback" => "__return_true", // Remove this line to require authentication.
    ]);

    register_rest_route("reorder", "get-master-url", [
        "methods" => WP_REST_Server::READABLE,
        "callback" => "get_master_url",
        "permission_callback" => "__return_true", // Remove this line to require authentication.
    ]);

    register_rest_route("reorder", "get-plugin-version", [
        "methods" => WP_REST_Server::READABLE,
        "callback" => "get_plugin_version",
        "permission_callback" => "__return_true", // Remove this line to require authentication.
    ]);
});

function get_plugin_version()
{
    return new WP_REST_Response(["version" => OMNI_VERSION]);
}

function validate_api_key(WP_REST_Request $request)
{
    $received_api_key = $request->get_param("api_key");
    $stored_api_key = get_option("omni_api_key");

    if ($received_api_key === $stored_api_key) {
        return rest_ensure_response(["valid" => true]);
    } else {
        return rest_ensure_response(["valid" => false]);
    }
}

function get_order_by_id($data)
{
    $order_id = $data["id"];
    $current_paths = GET_TABLES;
    $path = $current_paths[$order_id];
    if (!isset($path)) {
        return new WP_Error(
            "invalid_page_id",
            "The page ID does not exist in the current paths.",
            ["status" => 404]
        );
    }
    $page = [];
    $page["id"] = $path["id"];
    $page["url"] = $path["url"];
    $page["order"] = get_order_table($path["id"]);
    return new WP_REST_Response($page);
}

function check_and_get_tables()
{
    $tables = get_option("omni_table_file_paths");

    $modified = false;

    foreach ($tables as $key => $table) {
        if ($table["type"] === "html") {
            $path = $table["page"];
            if (!file_exists($path)) {
                unset($tables[$key]);
                $modified = true;
            }
        }
    }

    if ($modified) {
        update_option("omni_table_file_paths", $tables, false);
    }

    return $tables;
}

function get_tables()
{
    $omni_table_file_paths = GET_TABLES;

    return rest_ensure_response($omni_table_file_paths);
}

function get_order($data)
{
    $current_paths = GET_TABLES;
    $order = [];
    foreach ($current_paths as $path) {
        $page = [];
        $page["id"] = $path["id"];
        $page["url"] = $path["url"];
        $page["order"] = get_order_table($path["id"]);
        $order[$path["id"]] = $page;
    }
    return new WP_REST_Response($order);
}

function get_order_table($post_id)
{
    $order = [];
    $current_paths = GET_TABLES;
    $path_info = $current_paths[$post_id] ?? null;

    $dom = new DOMDocument();

    if ($path_info && $path_info["type"] === "html") {
        // Handling HTML file
        $htmlFilePath = $path_info["page"];
        if (!file_exists($htmlFilePath)) {
            error_log("File not found: " . $htmlFilePath);
            exit();
        }

        // Load the file's contents
        $htmlContent = file_get_contents($htmlFilePath);
        if ($htmlContent === false) {
            error_log("Failed to load file contents: " . $htmlFilePath);
            exit();
        }

        // Convert encoding
        $htmlContent = mb_convert_encoding(
            $htmlContent,
            "HTML-ENTITIES",
            "UTF-8"
        );

        // Load the HTML content into DOMDocument
        libxml_use_internal_errors(true);
        $dom->loadHTML($htmlContent);
        libxml_clear_errors();
        $xpath = new DOMXPath($dom);

        $omniItems = $xpath->query(
            "//*[contains(concat(' ', normalize-space(@class), ' '), ' omni-item ')]"
        );

        foreach ($omniItems as $i => $div) {
            $title = $div->getAttribute("name");
            $hidden = false;
            if (strpos($div->getAttribute("class"), "hidden") !== false) {
                $hidden = true;
            }

            if (!isset($title) || $title === "") {
                $omniTitle = $xpath->query(
                    ".//*[contains(concat(' ', normalize-space(@class), ' '), ' omni-title ')]",
                    $div
                );

                if ($omniTitle->length > 0) {
                    $title = trim($omniTitle->item(0)->textContent);
                }
            }
            $order[$i + 1] = [
                "title" => $title,
                "hidden" => $hidden,
            ];
        }
    } else {
        // Handling Elementor Page
        if (
            !\Elementor\Plugin::$instance->db->is_built_with_elementor($post_id)
        ) {
            return $order;
        }
        $page = \Elementor\Plugin::$instance->frontend->get_builder_content(
            $post_id
        );
        libxml_use_internal_errors(true);
        $dom->loadHTML(mb_convert_encoding($page, "HTML-ENTITIES", "UTF-8"));
        libxml_clear_errors();
        $xpath = new DOMXPath($dom);

        $omniItems = $xpath->query(
            "//*[contains(concat(' ', normalize-space(@class), ' '), ' omni-item ')]"
        );

        foreach ($omniItems as $i => $item) {
            $dataId = $item->getAttribute("data-id");
            $title = "";
            $omniTitle = $xpath->query(
                ".//*[contains(concat(' ', normalize-space(@class), ' '), ' elementor-heading-title ')]",
                $item
            );

            if ($omniTitle->length > 0) {
                $title = trim($omniTitle->item(0)->textContent);
            }
            $order[$i + 1] = ["id" => $dataId, "title" => $title];
        }
    }

    return $order;
}

function remove_page(WP_REST_Request $request)
{
    $page = $request->get_json_params();
    $remove_page_id = $page["id"] ?? null;

    $current_paths = GET_TABLES;

    if (!$remove_page_id) {
        return new WP_Error(
            "invalid_request",
            "Invalid or missing id in request.",
            ["status" => 400]
        );
    }
    if (isset($current_paths[$remove_page_id])) {
        unset($current_paths[$remove_page_id]);
        update_option("omni_table_file_paths", $current_paths, false);
        return rest_ensure_response([
            "message" => "Page successfully removed.",
        ]);
    }
    return new WP_Error(
        "mismatched_id",
        "The page id does not match any of the connected pages.",
        ["status" => 500]
    );
}

function reorder_table(WP_REST_Request $request)
{
    $current_paths = GET_TABLES;
    $page = $request->get_json_params();
    $post_id = $page["id"] ?? null;
    $doc = new DOMDocument();

    if (
        !$post_id ||
        !isset($page["order"]) ||
        !is_array($page["order"]) ||
        empty($page["order"])
    ) {
        return new WP_Error(
            "invalid_request",
            "Invalid or missing parameters in request.",
            ["status" => 400]
        );
    }

    $path_info = $current_paths[$post_id] ?? null;

    if (!$path_info) {
        return new WP_Error(
            "invalid_page_id",
            "The page ID does not exist in the current paths.",
            ["status" => 404]
        );
    }

    $orderedArray = object_to_array($page["order"]);

    if ($path_info && $path_info["type"] === "html") {
        // Handling HTML file
        $htmlFilePath = $path_info["page"];
        if (!file_exists($htmlFilePath)) {
            error_log("File not found: " . $htmlFilePath);
            return new WP_Error("file_not_found", "HTML file not found", [
                "status" => 404,
            ]);
        }

        // Extract file information
        $path_parts = pathinfo($htmlFilePath);

        // Define the backups folder path
        $backups_folder_path = $path_parts["dirname"] . "/backups";

        // Create the 'backups' folder if it doesn't exist
        if (
            !file_exists($backups_folder_path) &&
            !mkdir($backups_folder_path, 0777, true)
        ) {
            error_log("Failed to create backups folder: $backups_folder_path");
        }

        // Generate a timestamped backup file name
        $timestamp = date("Y-m-d_H-i-s");
        $backup_file_name =
            $path_parts["filename"] .
            "-" .
            $timestamp .
            "." .
            $path_parts["extension"];
        $backup_file_path = $backups_folder_path . "/" . $backup_file_name;

        // Copy the original file to the new backup file
        if (!copy($htmlFilePath, $backup_file_path)) {
            error_log("Failed to copy $htmlFilePath to $backup_file_path");
        }

        // Load the file's contents
        $htmlContent = file_get_contents($htmlFilePath);
        if ($htmlContent === false) {
            error_log("Failed to load file contents: " . $htmlFilePath);
            return new WP_Error("file_read_error", "Failed to read HTML file", [
                "status" => 500,
            ]);
        }

        // Decode the content before manipulation
        $htmlContent = html_entity_decode(
            $htmlContent,
            ENT_QUOTES | ENT_HTML5,
            "UTF-8"
        );

        // Convert encoding
        $htmlContent = mb_convert_encoding(
            $htmlContent,
            "HTML-ENTITIES",
            "UTF-8"
        );

        // Load the HTML content into DOMDocument
        libxml_use_internal_errors(true);
        $doc = new DOMDocument();
        $doc->loadHTML($htmlContent);
        libxml_clear_errors();

        $xpath = new DOMXPath($doc);
        $divs = $xpath->query(
            "//*[contains(concat(' ', normalize-space(@class), ' '), ' omni-item ')]"
        );

        if ($divs->length != count($orderedArray)) {
            return new WP_Error(
                "mismatched_counts",
                "The number of elements in the order array does not match the number of omni-item elements.",
                ["status" => 500]
            );
        }

        // Map original divs with their initial order
        $originalDivs = [];
        foreach ($divs as $index => $div) {
            $title = $div->getAttribute("name");
            if (!isset($title) || $title === "") {
                $omniTitle = $xpath->query(
                    ".//*[contains(concat(' ', normalize-space(@class), ' '), ' omni-title ')]",
                    $div
                );

                if ($omniTitle->length > 0) {
                    $title = trim($omniTitle->item(0)->textContent);
                }
            }

            $originalDivs[$title] = $div;
        }

        $orderedIndex = 0;
        $parent = $divs->item(0)->parentNode;
        $numChildren = $parent->childNodes->length;
        for ($i = 0; $i < $numChildren; $i++) {
            $child = $parent->childNodes[$i];
            if (
                $child->nodeType == XML_ELEMENT_NODE &&
                strpos($child->getAttribute("class"), "omni-item") !== false
            ) {
                // This is a section we want to replace
                $replacement = $originalDivs[
                    $orderedArray[$orderedIndex]["title"]
                ]->cloneNode(true);

                // Get current classes and convert to an array
                $classes = explode(" ", $replacement->getAttribute("class"));

                // Determine if we need to hide or show the element
                if ($orderedArray[$orderedIndex]["hidden"]) {
                    // Add 'hidden' class if not already present
                    if (!in_array("hidden", $classes)) {
                        $classes[] = "hidden";
                    }
                    // Add 'display: none;' style
                    $replacement->setAttribute("style", "display: none;");
                } else {
                    // Remove 'hidden' class if present
                    $classes = array_filter($classes, function ($class) {
                        return $class !== "hidden";
                    });
                    // Remove 'display: none;' style, if any, to revert to default or previous display style
                    // Check if there's an existing style attribute to preserve other style settings
                    $existingStyle = $replacement->getAttribute("style");
                    $updatedStyle = preg_replace(
                        "/display: none;?/",
                        "",
                        $existingStyle
                    );
                    $replacement->setAttribute("style", $updatedStyle);
                }

                // Set the modified classes back on the element
                $replacement->setAttribute("class", implode(" ", $classes));

                if ($replacement) {
                    $parent->replaceChild($replacement, $child);
                    $orderedIndex++;
                }
            }
        }

        // Save the updated HTML to a new file
        $html_content = $doc->saveHTML();

        // Ensure special characters are not encoded
        // 	    $html_content = str_replace(
        // 	        array('%7B', '%7D', '%7C'),
        // 	        array('{', '}', '|'),
        // 	        $html_content
        // 	    );

        file_put_contents($htmlFilePath, $html_content);
    } elseif (
        \Elementor\Plugin::$instance->db->is_built_with_elementor($post_id)
    ) {
        // Handling Elementor Page
        $elementor_data = get_post_meta($post_id, "_elementor_data", true);
        $elementor_data = json_decode($elementor_data, true);

        $resultJson = dfs_search_and_replace(
            $elementor_data,
            "omni-item",
            $orderedArray
        );
        if (is_wp_error($resultJson)) {
            return $resultJson;
        }

        update_post_meta(
            $post_id,
            "_elementor_data",
            wp_slash(json_encode($resultJson))
        );
        return rest_ensure_response([
            "message" => "Elementor page reordered successfully.",
        ]);
    } else {
        return new WP_Error(
            "unsupported_page_type",
            "The page is neither an HTML file nor built with Elementor.",
            ["status" => 500]
        );
    }
}

function dfs_search_and_replace(&$node, $target_class, $ordered_array)
{
    try {
        if (is_array($node) && array_key_exists("elements", $node)) {
            if (
                array_filter($node["elements"], function ($el) use (
                    $target_class
                ) {
                    return isset($el["settings"]["css_classes"]) &&
                        strpos(
                            $el["settings"]["css_classes"],
                            $target_class
                        ) !== false;
                })
            ) {
                $reordered = replace_elements(
                    $node["elements"],
                    $ordered_array
                );
                if (is_wp_error($reordered)) {
                    return $reordered;
                }
                $node["elements"] = $reordered;
                return $node;
            } else {
                foreach ($node["elements"] as &$el) {
                    $result = dfs_search_and_replace(
                        $el,
                        $target_class,
                        $ordered_array
                    );
                    if (is_wp_error($result)) {
                        return $result;
                    }
                    if ($result !== null) {
                        return $result;
                    }
                }
            }
        } elseif (is_array($node)) {
            foreach ($node as &$item) {
                $result = dfs_search_and_replace(
                    $item,
                    $target_class,
                    $ordered_array
                );
                if (is_wp_error($result)) {
                    return $result;
                }
                if ($result !== null) {
                    return $node;
                }
            }
        }
        return null;
    } catch (Exception $e) {
        return new WP_Error("caught_exception", $e->getMessage(), [
            "status" => 500,
        ]);
    }
}

function replace_elements(&$elements, $ordered_array)
{
    try {
        $omni_item_map = [];
        foreach ($elements as $element) {
            if (
                isset($element["settings"]["css_classes"]) &&
                strpos($element["settings"]["css_classes"], "omni-item") !==
                    false
            ) {
                $omni_item_map[$element["id"]] = $element;
            }
        }
        if (count($ordered_array) != count($omni_item_map)) {
            return new WP_Error(
                "mismatched_counts",
                "The number of elements in new order array does not match the number of omni-item elements on the page.",
                ["status" => 500]
            );
        }
        $ordered_index = 0;
        foreach ($elements as &$element) {
            if (
                isset($element["settings"]["css_classes"]) &&
                strpos($element["settings"]["css_classes"], "omni-item") !==
                    false
            ) {
                $el =
                    $omni_item_map[
                        $ordered_array[strval($ordered_index)]["id"]
                    ];
                if (isset($el)) {
                    $element = $el;
                }
                $ordered_index++;
            }
        }
        return $elements;
    } catch (Exception $e) {
        return new WP_Error("caught_exception", $e->getMessage(), [
            "status" => 500,
        ]);
    }
}

function omni_verify_api_key($request)
{
    $api_key = $request->get_header("x-api-key");
    $stored_api_key = get_option("omni_api_key");
    return $api_key === $stored_api_key;
}

// Function to set the API key
function set_api_key($key)
{
    update_option("omni_api_key", $key);
}

// Function to get the API key
function get_api_key()
{
    return get_option("omni_api_key");
}

function make_api_request($endpoint, $method = "GET", $body = null)
{
    $api_url = get_site_url() . "/wp-json/reorder/" . $endpoint;
    $api_key = get_api_key();
    $args = [
        "headers" => [
            "x-api-key" => $api_key,
        ],
    ];

    if ($method == "POST" && $body != null) {
        $args["method"] = "POST";
        $args["body"] = json_encode($body);
        $args["headers"]["Content-Type"] = "application/json";
    }

    $response = wp_remote_request($api_url, $args);

    if (is_wp_error($response)) {
        return "Error: " . $response->get_error_message();
    }

    return wp_remote_retrieve_body($response);
}

function test_master()
{
    $omni_backend_url = get_option("omni_backend_url");
    if (empty($omni_backend_url)) {
        wp_send_json_error("Omni backend URL is not set");
        return;
    }

    $ip = $_SERVER["REMOTE_ADDR"];
    $queryUrl = home_url($_SERVER["REQUEST_URI"]);
    $testId = wp_generate_password(16, true);

    $body = new stdClass();
    $body->url = $queryUrl;
    $body->ip = $ip;
    $body->reportType = "test";
    $body->testId = $testId;

    $url = $omni_backend_url . "/bigquery/test"; // Use the retrieved URL here
    $res = wp_remote_post($url, [
        "headers" => ["Content-Type" => "application/json; charset=utf-8"],
        "timeout" => 30,
        "body" => json_encode($body),
        "httpversion" => "1.0",
    ]);

    if (is_wp_error($res)) {
        wp_send_json_error("Error in request");
    }

    $responseBody = wp_remote_retrieve_body($res);
    $responseData = json_decode($responseBody);

    if ($responseData[0]->testId === $testId) {
        wp_send_json_success("Test successful");
    } else {
        wp_send_json_error("TestId mismatch");
    }
}

function get_master_url()
{
    $omni_backend_url = get_option("omni_backend_url");
    if (empty($omni_backend_url)) {
        wp_send_json_error("Omni backend URL is not set");
        return;
    }

    $body = new stdClass();
    $body->url = $omni_backend_url;

    wp_send_json_success($body);
    return;
}

function object_to_array($jsonObject)
{
    $dataArray = [];
    foreach ($jsonObject as $key => $value) {
        $index = intval($key) - 1;
        $dataArray[$index] = $value;
    }
    return $dataArray;
}

// Omni WP Proxy ------------------------------------------------------------

function omni_wp_settings_page()
{
    // Retrieve the current API key
    $api_key = get_option("omni_api_key");
    wp_enqueue_style(
        "font-awesome",
        "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/css/all.min.css"
    );
    ?>
    <div class="wrap" style="width: 60%; margin: 0 auto; font-size: 16px; margin-top: 8rem">
    <h1 style="text-align: center; color: #333; margin-bottom: 2rem">Omni WP Settings</h1>
    <form method="post" action="options.php">
      <?php settings_fields("omni-wp-settings-group"); ?>
      <?php do_settings_sections("omni-wp-settings-group"); ?>
      <table class="form-table">
        <tr valign="top">
          <th scope="row" style="width: min-content;display: flex;align-items: center;height: 70px;white-space: nowrap; padding-top: 0px; padding-bottom: 0px;">
              Master URL:
          </th>
          <td>
            <div class="display: flex; flex-direction: row; width: 100%">
              <input type="text" readonly id="omni_backend_url" name="omni_backend_url" value="<?php echo esc_attr(
                  get_option("omni_backend_url")
              ); ?>" style="width: 70%; padding: 5px; border-radius: 5px;"/>
              <button type="button" id="edit_button" onclick="unlockInput()" style="margin-left: 10px; background: #28a745; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; transition: opacity 0.3s ease-in-out; cursor: pointer;"><i class="fas fa-pen"></i></button>
              <button type="button" id="reset_button" onclick="resetInput()" style="margin-left: 10px; background: #dc3545; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; transition: opacity 0.3s ease-in-out; cursor: pointer;"><i class="fas fa-sync-alt"></i></button>
              </div>
          </td>
        </tr>
      </table>
      <div style="display: flex; align-items: center; justify-content: space-evenly; margin-top: 10px;">
        <button type="submit" class="button button-primary" name="submit" id="submit" style="cursor: pointer; background: #28a745; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; transition: opacity 0.3s ease-in-out;">Save Changes</button>
        <div style="display: flex; align-items: center;  gap: 1rem">
          <button type="button" id="test_button" class="button button-primary" onclick="runTest()" style="background: #007bff; color: #fff; border: none; padding: 5px 10px; border-radius: 5px; transition: opacity 0.3s ease-in-out;">Test Master URL</button>
          <span id="test_result" style="font-size: 1.2em;"></span>
        </div>
      </div>
    </form>
    <!-- API Key Section -->
    <div style="margin: 8rem auto 0 auto; text-align: center;">
      Your API Key: <?= esc_html($api_key) ?>
    </div>
  </div>
   <script type="text/javascript">
    function runTest() {
        var testResultElement = document.getElementById('test_result');
        // Show spinner icon when the test starts
        testResultElement.innerHTML = '<span class="fas fa-spinner fa-spin"></span>';

        jQuery.ajax({
            url: ajaxurl,
            type: 'POST',
            data: {
                action: 'test_master'
            },
            success: function(response) {
        console.log(response);
                if (response.success) {
                    testResultElement.innerHTML = '<span class="fas fa-check" style="color: green;"></span>';
                } else {
                    testResultElement.innerHTML = `<div><span class="fas fa-times" style="color: red;"></span> ${response.data ? response.data : 'Something went wrong, could not find the record'}</div>`;
                }
            },
            error: function(response) {
                console.error('Error:', response);
                // Optionally update the icon or message in case of an error
                testResultElement.innerHTML = `<span class="fas fa-exclamation-triangle" style="color: orange;"> ${response ? response : 'Something went wrong, could not find the record'}</span>`;
            }
        });
    }

        function unlockInput() {
            document.getElementById("omni_backend_url").readOnly = false;
            document.getElementById("omni_backend_url").focus();
        }

        function resetInput() {
            document.getElementById("omni_backend_url").value = "<?php echo esc_attr(
                get_option("omni_backend_url")
            ); ?>";
            document.getElementById("omni_backend_url").readOnly = true;
        }
    </script>
    <?php
}

function register_omni_wp_settings()
{
    register_setting("omni-wp-settings-group", "omni_backend_url");
}
add_action("admin_init", "register_omni_wp_settings");

// Add Endpoint
// ENDPOINT : example.com/wp-json/proxy/v1/data/
add_action("rest_api_init", function () {
    register_rest_route("proxy/v1", "/data", [
        "methods" => "POST",
        "callback" => "run_proxy_req",
    ]);
});

function run_proxy_req(WP_REST_Request $request)
{
    $body = json_decode($request->get_body());
    $url =
        get_option("omni_backend_url") .
        "/bigquery/data?" .
        $_SERVER["QUERY_STRING"];
    $ip = $_SERVER["REMOTE_ADDR"];
    $page_id = get_queried_object_id();

    if (!isset($_GET["ApiKey"])) {
        return new WP_REST_Response("ApiKey not found", 400);
    }

    if (!is_array($body)) {
        return new WP_REST_Response('$body is not array', 400);
    }

    foreach ($body as &$item) {
        $item->url = $url;
        $item->ip = $ip;
        $item->pageId = $page_id;
    }

    $response = wp_remote_post($url, [
        "headers" => [
            "Content-Type" => "application/json; charset=utf-8",
            "ApiKey" => $_GET["ApiKey"],
        ],
        "timeout" => 30,
        "body" => json_encode($body),
        "httpversion" => "1.0",
    ]);

    if (is_wp_error($response)) {
        return new WP_REST_Response("Proxy request failed", 401);
    }
}

function enqueue_custom_footer_script() {
    $omni_table_file_paths = GET_TABLES;
    if (!$omni_table_file_paths || !is_array($omni_table_file_paths)) {
        return;
    }

    // Always enqueue the necessary scripts
    wp_enqueue_script("jquery");
    wp_enqueue_script(
        "outbound-click-script",
        plugins_url("scripts/outbound-click-script.js", __FILE__),
        ["jquery"],
        "1.0",
        true
    );

    $current_url = $_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"];
    $current_base_url = strtok($current_url, "?");
    
    // Try to find the matching order, first by exact URL match
    $matching_order = null;
    foreach ($omni_table_file_paths as $path) {
        if (isset($path["url"]) && $current_base_url === $path["url"]) {
            $matching_order = get_order_table($path["id"]);
            break;
        }
    }

    // If no exact match found, try to find by content path
    if (!$matching_order) {
        $actual_content_path = $_SERVER["SCRIPT_FILENAME"];
        foreach ($omni_table_file_paths as $path) {
            if (isset($path["page"]) && strpos($actual_content_path, $path["page"]) !== false) {
                $matching_order = get_order_table($path["id"]);
                break;
            }
        }
    }

    // If still no match, provide an empty array to prevent undefined errors
    if (!$matching_order) {
        $firstKey = key($omni_table_file_paths);
        $matching_order = get_order_table($firstKey);
    }

    wp_localize_script("outbound-click-script", "scriptParams", [
        "ajaxUrl" => admin_url("admin-ajax.php"),
        "referer" => isset($_SERVER["HTTP_REFERER"]) ? $_SERVER["HTTP_REFERER"] : "unknown",
        "OmniItemsOrder" => $matching_order
    ]);

    wp_enqueue_script(
        "custom-footer-script",
        plugins_url("scripts/custom-footer-script.js", __FILE__),
        ["jquery"],
        "1.0",
        true
    );

    wp_localize_script("custom-footer-script", "scriptParams", [
        "ajaxUrl" => admin_url("admin-ajax.php"),
        "currentUrl" => home_url($_SERVER["REQUEST_URI"]),
        "ipAddress" => $_SERVER["REMOTE_ADDR"],
        "referer" => isset($_SERVER["HTTP_REFERER"]) ? $_SERVER["HTTP_REFERER"] : "unknown",
        "OmniItemsOrder" => $matching_order
    ]);
}

add_action("wp_header", "enqueue_custom_footer_script");

add_action("wp_ajax_forward_to_external", "handle_forward_to_external");
add_action("wp_ajax_nopriv_forward_to_external", "handle_forward_to_external");

function handle_forward_to_external()
{
    $payload = $_POST["payload"];
    $ip = $_SERVER["REMOTE_ADDR"];
    $payload["ip"] = $ip;
    $url = get_option("omni_backend_url") . "/bigquery/query";

    $response = wp_remote_post($url, [
        "headers" => ["Content-Type" => "application/json; charset=utf-8"],
        "body" => json_encode($payload),
        "method" => "POST",
        "data_format" => "body",
    ]);

    if (is_wp_error($response)) {
        wp_send_json_error($response->get_error_message(), 400);
    } else {
        wp_send_json_success("Data forwarded successfully");
    }

    wp_die();
}

add_action("wp_ajax_handle_outbound_click", "handle_outbound_click");
add_action("wp_ajax_nopriv_handle_outbound_click", "handle_outbound_click");

function handle_outbound_click()
{
    $payload = $_POST["payload"];
    $ip = $_SERVER["REMOTE_ADDR"];
    $payload["ip"] = $ip;
    $payload["outbound_click_url"] = $payload["href"];
    $payload["event"] = "outbound_click";

    $url = get_option("omni_backend_url") . "/bigquery/query";
    $res = wp_remote_post($url, [
        "headers" => ["Content-Type" => "application/json; charset=utf-8"],
        "timeout" => 30,
        "body" => json_encode($payload),
        "httpversion" => "1.0",
    ]);

    if (is_wp_error($res)) {
        wp_send_json_error("Error in tracking outbound click.");
    } else {
        wp_send_json_success("Outbound click tracked successfully.");
    }

    wp_die();
}

add_action("wp_ajax_test_master", "testMaster");
add_action("wp_ajax_nopriv_test_master", "testMaster");

function testMaster()
{
    $ip = $_SERVER["REMOTE_ADDR"];
    $queryUrl = home_url($_SERVER["REQUEST_URI"]);
    $testId = wp_generate_password(16, true);

    $body = new stdClass();
    $body->url = $queryUrl;
    $body->ip = $ip;
    $body->reportType = "test";
    $body->testId = $testId;

    $url = get_option("omni_backend_url") . "/bigquery/test";
    $res = wp_remote_post($url, [
        "headers" => ["Content-Type" => "application/json; charset=utf-8"],
        "timeout" => 30,
        "body" => json_encode($body),
        "httpversion" => "1.0",
    ]);

    if (is_wp_error($res)) {
        wp_send_json_error("Error in request");
    }

    $responseBody = wp_remote_retrieve_body($res);
    $responseData = json_decode($responseBody);

    if ($responseData[0]->testId === $testId) {
        wp_send_json_success("Test successful");
    } else {
        wp_send_json_error("TestId mismatch");
    }
}

function register_my_plugin_rest_routes()
{
    register_rest_route("proxy/v1", "/forward_to_external", [
        "methods" => "POST",
        "callback" => "handle_forward_to_external_html",
    ]);

    register_rest_route("proxy/v1", "/handle_outbound_click", [
        "methods" => "POST",
        "callback" => "handle_outbound_click_html",
    ]);

    register_rest_route("proxy/v1", "/get_omni_items_order", [
        "methods" => "POST",
        "callback" => "get_omni_items_order",
    ]);
}
add_action("rest_api_init", "register_my_plugin_rest_routes");

function handle_forward_to_external_html(WP_REST_Request $request)
{
    $payload = $request->get_json_params();
    $url = get_option("omni_backend_url") . "/bigquery/query";
    $ip = $_SERVER["REMOTE_ADDR"];
    $payload["ip"] = $ip;

    $response = wp_remote_post($url, [
        "headers" => ["Content-Type" => "application/json; charset=utf-8"],
        "body" => json_encode($payload),
        "method" => "POST",
        "data_format" => "body",
    ]);

    if (is_wp_error($response)) {
        return new WP_Error(
            "rest_forward_failed",
            "Forward to external failed",
            ["status" => 400]
        );
    }

    return rest_ensure_response("Data forwarded successfully");
}

function handle_outbound_click_html(WP_REST_Request $request)
{
    $payload = $request->get_json_params();
    $url = get_option("omni_backend_url") . "/bigquery/query";

    $ip = $_SERVER["REMOTE_ADDR"];
    $payload["ip"] = $ip;
    $payload["outbound_click_url"] = $payload["href"];
    $payload["event"] = "outbound_click";

    $response = wp_remote_post($url, [
        "headers" => ["Content-Type" => "application/json; charset=utf-8"],
        "body" => json_encode($payload),
        "timeout" => 30,
        "httpversion" => "1.0",
    ]);

    if (is_wp_error($response)) {
        return new WP_Error(
            "rest_click_tracking_failed",
            "Error in tracking outbound click.",
            ["status" => 400]
        );
    }

    return rest_ensure_response("Outbound click tracked successfully.");
}

function get_omni_items_order(WP_REST_Request $request)
{
    $data = $request->get_json_params();
    $currentUrl = $data["url"];
    $omniTableFilePaths = GET_TABLES;

    foreach ($omniTableFilePaths as $path) {
        if (isset($path["url"]) && $currentUrl === $path["url"]) {
            $omniItemsOrder = get_order_table($path["id"]);
            return rest_ensure_response(["omniItemsOrder" => $omniItemsOrder]);
        }
    }

    return rest_ensure_response(["omniItemsOrder" => "Not found"]);
}

function update_html_files_with_new_script()
{
    $current_paths = GET_TABLES;

    foreach ($current_paths as $path) {
        if ($path["type"] === "html") {
            $html_file_path = $path["page"];
            if (file_exists($html_file_path)) {
                // Load the HTML content
                $html_content = file_get_contents($html_file_path);
                $html_content = html_entity_decode(
                    $html_content,
                    ENT_QUOTES | ENT_HTML5,
                    "UTF-8"
                );

                $doc = new DOMDocument();
                @$doc->loadHTML(
                    mb_convert_encoding($html_content, "HTML-ENTITIES", "UTF-8")
                );

                // Remove existing script params, homeUrl scripts, and our plugin scripts
                $scripts = $doc->getElementsByTagName("script");
                for ($i = $scripts->length - 1; $i >= 0; $i--) {
                    $script = $scripts->item($i);
                    $content = $script->textContent;
                    $src = $script->getAttribute("src");
                    
                    // Remove by content (for inline scripts)
                    if (strpos($content, 'var scriptParams =') !== false || 
                        strpos($content, 'window.homeUrl =') !== false) {
                        $script->parentNode->removeChild($script);
                        continue;
                    }
                    
                    // Remove by src (for external scripts)
                    $plugin_scripts = [
                        "outbound-click-script.js",
                        "custom-footer-script.js", 
                        "html-file-script.js"
                    ];
                    foreach ($plugin_scripts as $plugin_script) {
                        if (strpos($src, $plugin_script) !== false) {
                            $script->parentNode->removeChild($script);
                            break;
                        }
                    }
                }

                // Get the body element
                $body = $doc->getElementsByTagName('body')->item(0);
                if (!$body) {
                    error_log('No body element found in HTML file');
                    continue;
                }

                // Prepare the localized script data
                $script_params = [
                    'ajaxUrl' => admin_url('admin-ajax.php'),
                    'currentUrl' => home_url($_SERVER['REQUEST_URI']),
                    'ipAddress' => $_SERVER['REMOTE_ADDR'],
                    'referer' => isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : 'unknown',
                    'OmniItemsOrder' => get_order_table($path['id'])
                ];

                // Add localized script data and homeUrl in a single script tag
                $combinedScript = $doc->createElement('script');
                $combinedScript->setAttribute('type', 'text/javascript');
                $combinedScript->textContent = 
                    'var scriptParams = ' . json_encode($script_params) . ';' . 
                    'window.homeUrl = "' . home_url() . '";';
                $body->appendChild($combinedScript);

                // Add jQuery if it's not already present
                $hasJquery = false;
                for ($i = 0; $i < $scripts->length; $i++) {
                    $src = $scripts->item($i)->getAttribute("src");
                    if (strpos($src, "jquery") !== false) {
                        $hasJquery = true;
                        break;
                    }
                }

                if (!$hasJquery) {
                    $jqueryScript = $doc->createElement("script");
                    $jqueryScript->setAttribute(
                        "src",
                        "https://code.jquery.com/jquery-latest.min.js"
                    );
                    $jqueryScript->setAttribute("crossorigin", "anonymous");
                    $doc->getElementsByTagName("head")
                        ->item(0)
                        ->appendChild($jqueryScript);
                }

                // Add our three scripts in order
                $outboundScript = $doc->createElement("script");
                $outboundScript->setAttribute(
                    "src",
                    plugins_url("scripts/outbound-click-script.js", __FILE__)
                );
                $outboundScript->setAttribute("defer", "true");
                $body->appendChild($outboundScript);

                $footerScript = $doc->createElement("script");
                $footerScript->setAttribute(
                    "src",
                    plugins_url("scripts/custom-footer-script.js", __FILE__)
                );
                $footerScript->setAttribute("defer", "true");
                $body->appendChild($footerScript);

                $htmlScript = $doc->createElement("script");
                $htmlScript->setAttribute(
                    "src",
                    plugins_url("scripts/html-file-script.js", __FILE__)
                );
                $htmlScript->setAttribute("defer", "true");
                $body->appendChild($htmlScript);

                // Save the updated HTML
                $html_content = $doc->saveHTML();
                file_put_contents($html_file_path, $html_content);
            }
        }
    }
}