function getOmniItemsOrder() {
  // Get all elements with class 'omni-item'
  const omniItems = document.getElementsByClassName("omni-item");

  // Create an object instead of an array
  const items = Array.from(omniItems).reduce((acc, item, index) => {
    acc[index + 1] = {
      title: item.getAttribute("name"),
      hidden: item.classList.contains("hidden") ? "true" : "false",
    };
    return acc;
  }, {});

  return items;
}

jQuery(document).ready(function ($) {
  // Listen to all clicks for logging
  $(document).on("click mousedown contextmenu touchend", "*", function (event) {
    const element = $(this);

    // Only process further if element has href
    var href = element.attr("href");
    if (!href) return;

    var target = element.attr("target");

    if (
      href.indexOf(location.hostname) === -1 &&
      !href.startsWith("#") &&
      !href.startsWith("mailto:") &&
      !href.startsWith("tel:")
    ) {
      if (
        event.type === "click" ||
        event.type === "mousedown" ||
        event.type === "contextmenu" ||
        event.type === "touchend"
      ) {
        if (event.type === "mousedown" && event.which !== 2) {
          return;
        }

        var data = {
          action: "handle_outbound_click",
          ip: scriptParams.ipAddress,
          href: href,
          target: target,
          OmniItemsOrder: getOmniItemsOrder(),
          url: scriptParams.currentUrl,
          event: "outbound_click",
          referer: scriptParams.referer,
        };

        // For click/touchend events that aren't opening in new tab
        if (
          (event.type === "click" || event.type === "touchend") &&
          (!target || target !== "_blank")
        ) {
          event.preventDefault();

          jQuery.ajax({
            url: scriptParams.ajaxUrl,
            method: "POST",
            data: {
              action: "handle_outbound_click",
              payload: data,
            },
            complete: function () {
              window.location.href = href;
            },
          });
        } else {
          // For other events or new tab opens, just send the request
          jQuery.post(scriptParams.ajaxUrl, {
            action: "handle_outbound_click",
            payload: data,
          });
        }
      }
    }
  });
});
