jQuery(document).ready(function ($) {
  var pageId = $("input.pageId").val();
  var originalOrder = {};
  $("#sortableTable").sortable().disableSelection();

  // Save the original order when the page loads
  $("#sortableTable .sortableRow").each(function () {
    var id = $(this).find("input.id").val();
    var title = $(this).find("input.title").val();
    var hidden = !$(this).find("input.itemVisibilityCheckbox").is(":checked");
    var index = $(this).index();
    originalOrder[index + 1] = {
      id,
      title,
      hidden,
    };
  });

  $('input[type="submit"]').click(function (e) {
    e.preventDefault();

    var order = {};
    $("#sortableTable .sortableRow").each(function () {
      var id = $(this).find("input.id").val();
      var title = $(this).find("input.title").val();
      var hidden = !$(this).find("input.itemVisibilityCheckbox").is(":checked");
      var index = $(this).index();
      order[index + 1] = {
        id,
        title,
        hidden,
      };
    });

    var json = {
      id: pageId,
      order,
    };

    $.ajax({
      url: omniSettings.apiUrl, // Use the API URL from the server
      type: "POST",
      headers: {
        "x-api-key": omniSettings.apiKey, // Include the API key in the request headers
      },
      data: JSON.stringify(json),
      contentType: "application/json; charset=utf-8",
      dataType: "json",
      success: function (data) {
        alert("Order saved successfully!");
        location.reload();
      },
      error: function (jqXHR) {
        var errorMessage = "Error: ";
        if (jqXHR.responseJSON && jqXHR.responseJSON.message) {
          errorMessage += jqXHR.responseJSON.message;
        } else {
          errorMessage += "An unknown error occurred.";
        }
        alert(errorMessage);
      },
    });
  });

  // Reset the order when the "Reset changes" button is clicked
  $("#resetOrder").click(function () {
    $("#sortableTable .sortableRow").each(function (index) {
      var $row = $(this);
      var id = $row.find("input.id").val();
      var title = $row.find("input.title").val();
      var visibilityCheckbox = $row.find("input.itemVisibilityCheckbox");

      var shouldBeChecked = !originalOrder[index + 1].hidden;

      if (
        originalOrder[index + 1].id !== id ||
        originalOrder[index + 1].title !== title ||
        visibilityCheckbox.is(":checked") !== shouldBeChecked
      ) {
        $row.find(".id").val(originalOrder[index + 1].id);
        $row.find(".title").val(originalOrder[index + 1].title);
        $row.find("strong").text(originalOrder[index + 1].title);
        $row.find("span.orderIndex").text(index + 1);
        visibilityCheckbox.prop("checked", shouldBeChecked);
      }
    });
    // Trigger the update event of sortable to refresh the order
    $("#sortableTable").trigger("sortupdate");
  });
});
